require 'roqua/support/log_wrapper'
require 'logger'
require 'stringio'

module Roqua
  describe LogWrapper do
    before { allow(Roqua).to receive(:appname).and_return('roqua-support-testsuite') }

    let(:logstream)  { StringIO.new }
    let(:logger)     { Logger.new(logstream) }
    let(:logwrapper) { LogWrapper.new(logger) }

    def log
      logstream.string
    end

    describe '#add' do
      it 'writes event name to log' do
        logwrapper.add :info, "testevent"
        expect(log).to include("testevent {}\n")
      end

      it 'writes given parameters as json hash' do
        logwrapper.add :info, "testevent", extra: 'params', go: 'here'
        expect(log).to include('testevent {"extra":"params","go":"here"}' + "\n")
      end

      it 'escapes newline characters in params' do
        logwrapper.add :info, "testevent", param: "this\nshould not have newlines"
        expect(log).to include('testevent {"param":"this\nshould not have newlines"')
      end
    end

    describe '#lifecycle' do
      it 'logs the start and finish lifecycle of a block' do
        logwrapper.lifecycle 'testevent', extra: 'params' do
          1 + 1
        end
        expect(log).to include('testevent:started {"extra":"params"}')
        expect(log).to match(/testevent:finished.*"extra":"params"/)
      end

      it 'logs the duration of the block with the finished event' do
        logwrapper.lifecycle('testevent') { 1 + 1 }
        expect(log).to match(/testevent:finished.*"duration":/)
      end

      it 'returns the value returned by the block' do
        expect(logwrapper.lifecycle('testevent') { 1 + 1 }).to eq 2
      end

      it 'logs the start and failure of a block if it raises' do
        logwrapper.lifecycle 'testevent' do
          raise StandardError, "Foo"
        end rescue nil
        expect(log).to include('testevent:started')
        expect(log).to include('testevent:failed {"exception":"StandardError","message":"Foo"}')
      end

      it 'reraises the exception' do
        expect {
          logwrapper.lifecycle 'testevent' do
            raise "Foo"
          end
        }.to raise_error('Foo')
      end
    end
  end
end
